/* * Copyright (C) 2021 Advanced Micro Devices. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file gpuvsmi.h
 *
 * GPU SMI Library Interface
 */

#ifndef __GPUVSMI_H__
#define __GPUVSMI_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stdbool.h>

#include "smi_lib_types.h"

/*---------------------------------------------------------------------------*/
/** @defgroup init     Library Initialization                                */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Initializes the library and the internal software
 *                  structures.
 *
 *  \note           The library will attempt to initialize all AMD GPU devices
 *                  in the system, but this may fail if the library is called
 *                  from a user without sufficient privileges.
 *                  The library will attempt to initialize in privileged mode
 *                  or initialize in non-privileged mode as a fallback.
 *                  Privileged mode can modify the driver configuration.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Library loads successfully
 *                  * -::SMI_ERR_NOT_FOUND - No AMD devices are found
 *                  * -::SMI_ERR_NO_PERM - Library load failed, insufficient permissions
 *                  * -::SMI_ERR_API_FAILED - Driver loading or GPU initialization failed
 */
int gpuvsmi_init(void);

/**
 *  \brief          Initializes the library and the internal software
 *                  structures.
 *
 *  \note           The library will attempt to initialize all AMD GPU devices
 *                  in the system, but this may fail if the library is called
 *                  from a user without sufficient privileges. The library can
 *                  be initialized in privileged or non-privileged mode.
 *                  Privileged mode can modify the driver configuration.
 *
 *  \param [in]     is_privileged - Indicates if the library should be run in
 *                  privileged mode.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Library loads successfully
 *                  * -::SMI_ERR_NOT_FOUND - No AMD devices are found
 *                  * -::SMI_ERR_NO_PERM - Library load failed, insufficient permissions
 *                  * -::SMI_ERR_API_FAILED - Driver loading or GPU initialization failed
 */
int gpuvsmi_init_flags(bool is_privileged);

/**
 *  \brief          The library frees all associated with the library in the
 *                  current process.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Library unloads successfully
 *                  * -::SMI_ERR_API_FAILED - Library unload fail
 */
int gpuvsmi_fini(void);

/** @}  */
/*---------------------------------------------------------------------------*/
/** @defgroup discovery     Device Discovery                                 */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the number of AMD GPU devices in the system.
 *
 *  \param [out]    device_count - Reference to return
 *                  the number of devices detected in the system.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_device_count(unsigned int *device_count);

/**
 *  \brief          Returns a list of AMD GPU devices in the system.
 *
 *  \note           This function fills the user-provided buffer with
 *                  GPU device handles. The device handles returned are used
 *                  to instantiate the rest of device queries in the library.
 *                  If the buffer is not large enough the call will fail.
 *
 *  \param [in,out] device_count - As input, the size of the provided buffer.
 *                  As output, number of device handles in the buffer.
 *                  Parameter must be allocated by user.
 *
 *  \param [out]    devices - Reference to list of device handles returned by
 *                  the library. Buffer must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are invalid
 *                  * -::SMI_ERR_NOMEM - Provided buffer is not large enough
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_devices(unsigned int *device_count, struct smi_device_handle *devices);

/**
 *  \brief          Returns device handle for the given BDF
 *
 *  \param [in]     bdf - BDF of the device
 *
 *  \param [out]    dev - Reference to the device handle.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are invalid
 *                  * -::SMI_ERR_NOT_FOUND - Device cannot be found
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_device_handle(union smi_bdf bdf, struct smi_device_handle *dev);

/**
 *  \brief          Returns BDF of the given device
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    bdf - Reference to BDF. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are invalid
 *                  * -::SMI_ERR_NOT_FOUND - Device cannot be found
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_device_bdf(struct smi_device_handle dev, union smi_bdf *bdf);

/**
 *  \brief          Returns the UUID of the device
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in,out] uuid_length - Length of the uuid string. As input, must be
 *                  equal or greater than SMI_GPU_UUID_SIZE and be allocated by
 *                  user. As output it is the length of the uuid string.
 *
 *  \param [out]    uuid - Pointer to string to store the UUID. Must be
 *                  allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are invalid
 *                  * -::SMI_ERR_NOT_FOUND - Device cannot be found
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_device_uuid(struct smi_device_handle dev, unsigned int *uuid_length,
			    char *uuid);

/** @}  */
/*---------------------------------------------------------------------------*/
/** @defgroup swversion     SW Version Information                           */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the driver version information
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in,out] length - As input parameter length of the user allocated
 * 	                string buffer. As output parameter length of the returned
 *                  string buffer.
 *
 *  \param [out]    version - Version information in string format. Must be
 *                  allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NOT_FOUND - Device cannot be found
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_driver_version(struct smi_device_handle dev, int *length, char *version);

/** @}  */
/*---------------------------------------------------------------------------*/
/** @defgroup asicinfo     ASIC & Board Static Information                   */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the ASIC information for the device.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to static asic information structure.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_asic_info(struct smi_device_handle dev, struct smi_asic_info *info);

/**
 *  \brief          Returns the device bus information.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to bus information structure. Must be
*                   allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_bus_info(struct smi_device_handle dev, struct smi_bus_info *info);

/**
 *  \brief          Returns the power caps as currently configured in the
 *                  system.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to power information structure. Must be
 *                  allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_power_info(struct smi_device_handle dev, struct smi_power_info *info);

/**
 *  \brief          Returns the device capabilities as currently configured in
 *                  the system
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to caps information structure. Must be
 *                  allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_caps_info(struct smi_device_handle dev, struct smi_gpu_caps *info);

/** @}  */
/*---------------------------------------------------------------------------*/
/** @defgroup ucodeinfo     Microcode & VBIOS queries                        */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the static information for the vBIOS on the device.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to static vBIOS information.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_vbios_info(struct smi_device_handle dev, struct smi_vbios_info *info);

/**
 *  \brief          Returns the microcode versions running on the device.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to the ucode info. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_ucode_info(struct smi_device_handle dev, struct smi_ucode_info *info);

/** @}  */
/*---------------------------------------------------------------------------*/
/** @defgroup gpumon     GPU Monitoring                                      */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the current usage of the GPU engines (GFX, MM and MEM).
 *                  Each usage is reported as a percentage from 0-100%.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to the gpu usage structure. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_gpu_activity(struct smi_device_handle dev, struct smi_engine_usage *info);

/**
 *  \brief          Returns the current power and voltage of the GPU.
 *                  The voltage is in units of mV and the power in units of W.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to the gpu power structure. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_power_measure(struct smi_device_handle dev, struct smi_power_measure *info);

/**
 *  \brief          Returns the measurements of the clocks in the GPU
 *                  for the GFX and multimedia engines and Memory. This call
 *                  reports the averages over 1s in MHz.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in]     domain - Enum representing the domain to query. It should
 *		    be one on the smi_clk_domain
 *
 *  \param [out]    info - Reference to the gpu clock structure.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_clock_measure(struct smi_device_handle dev, enum smi_clock_domain domain,
			      struct smi_gpu_clock_measure *info);

/**
 *  \brief          Returns thermal measurements of the GPU.
 *                  The results are in °C.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in]     domain - Enum representing the domain to query. It should
 *                  be one in the smi_thermal_domain
 *
 *  \param [out]    info - Reference to the temperature measured.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_thermal_measure(struct smi_device_handle dev, enum smi_thermal_domain domain,
				struct smi_gpu_thermal_measure *info);

/**
 *  \brief          Returns thermal limit of the GPU.
 *                  The results are in °C.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in]     domain - Enum representing the domain to query. It should
 *                  be one in the smi_thermal_domain
 *
 *  \param [out]    info - Reference to the temperature limit.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_thermal_limit(struct smi_device_handle dev, enum smi_thermal_domain domain,
				int *limit);

/**
 *  \brief          Returns power limit of the GPU.
 *                  The results are in W.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    info - Reference to the power limit.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_power_limit(struct smi_device_handle dev, int *limit);

/**
 *  \brief          Returns the current PCIe metrics for the GPU.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    lanes - Reference to current number of lanes used.
 *                  Must be allocated by user.
 *
 *  \param [out]    speed - Reference to the current PCIe speed in MT/s.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_pcie_link_status(struct smi_device_handle dev, uint8_t *lanes,
				 uint32_t *speed);

/**
 *  \brief          Returns the framebuffer usage (both total and used memory)
 *                  in MegaBytes.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    fb_total - Returned total framebuffer size in MegaBytes.
 *                  Must be allocated by user.
 *
 *  \param [out]    fb_used - Returned used framebuffer memory in MegaBytes.
 *                  Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_fb_usage(struct smi_device_handle dev, uint32_t *fb_total, uint32_t *fb_used);

/** @}  */
/*---------------------------------------------------------------------------*/
/** @defgroup gpumon     Power Management                                    */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the supported frequency target range for the
 *                  specified clock domain.
 *
 *  \note           This API is Not Supported
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in]     domain - Clock domain for which to get supported frequency
 *                  range.
 *
 *  \param [out]    soft_min - Minimal value of target frequency for specified
 *                  domain in MHz. Must be allocated by user.
 *
 *  \param [out]    soft_max - Maximal value of target frequency for specified
 *                  domain in MHz. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialize
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_target_frequency_supported_range(struct smi_device_handle dev,
				enum smi_clock_domain domain,
				uint32_t *soft_min, uint32_t *soft_max);


/**
 *  \brief          Returns the current frequency target range for the
 *                  specified clock domain.
 *
 *  \note           This API is Not Supported
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in]     domain - Clock domain for which to get current frequency
 *                  range.
 *
 *  \param [out]    soft_min - Minimal target frequency for specified clock
 *                  domain in MHz. Must be allocated by user.
 *
 *  \param [out]    soft_max - Maximal target frequency for specified clock
 *                  domain in MHz. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialize
 *                  * -::SMI_ERR_RETRY - Device is busy. Please retry
 *                  * -::SMI_ERR_INVAL - Parameters are not valid or NULL
 *                  * -::SMI_ERR_IO - Device is in an unrecoverable state
 *                  * -::SMI_ERR_NOT_INIT - Device is uninitialized
 *                  * -::SMI_ERR_API_FAILED - Other errors
 */
int gpuvsmi_get_target_frequency_current_range(struct smi_device_handle dev,
				enum smi_clock_domain domain,
				uint32_t *soft_min, uint32_t *soft_max);

/** @}  */
/*---------------------------------------------------------------------------*/
/**  @defgroup processinfo     Process information                           */
/*---------------------------------------------------------------------------*/
/** @{  */

/**
 *  \brief          Returns the list of processes running on a given GPU.
 *
 *  \note           The user provides a buffer to store the list and the
 *                  maximum number of processes that can be returned. If the user
 *                  sets max_processes to 0, the total number of processes will be
 *                  returned.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [out]    list - Reference to a user-provided buffer where the process
 *                  list will be returned. This buffer must contain at least
 *                  max_processes entries of type smi_process_handle. Must be allocated
 *                  by user.
 *
 *  \param [in,out] max_processes - Reference to the size of the list buffer in
 *                  number of elements. Returns the return number of elements
 *                  in list or the number of running processes if equal to 0.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - The parameters are not valid or NULL
 *                  * -::SMI_ERR_NOMEM - Provided buffer is not large enough
 *                  * -::SMI_ERR_NOT_SUPPORTED - API not supported
 */
int gpuvsmi_get_process_list(struct smi_device_handle dev, smi_process_handle *list,
			     uint32_t *max_processes);

/**
 *  \brief          Returns the process information of a given process.
 *
 *  \param [in]     dev - device which to query
 *
 *  \param [in]     process - handle of process to query.
 *
 *  \param [out]    info - Reference to a process information structure where to return
 *                  information. Must be allocated by user.
 *
 *  \return
 *                  *  ::SMI_SUCCESS - Successful
 *                  * -::SMI_ERR_NO_PERM - Library was not initialized
 *                  * -::SMI_ERR_INVAL - The parameters are not valid or NULL
 *                  * -::SMI_ERR_NOT_SUPPORTED - API not supported
 */
int gpuvsmi_get_process_info(struct smi_device_handle dev, smi_process_handle process,
			     struct smi_process_info *info);

#ifdef __cplusplus
}
#endif

#endif // __GPUVSMI_H__
