/* * Copyright (C) 2021 Advanced Micro Devices. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file smi_lib_types.h
 *
 * GPU SMI Library Types
 */

#ifndef __SMI_LIB_TYPES_H__
#define __SMI_LIB_TYPES_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "smi_lib_ret.h"

#ifndef __KERNEL__
#include <stdint.h>
#include <stdbool.h>
#endif

#ifndef __linux__
#ifdef _WIN64
#pragma pack(push, 8)
#else
#pragma pack(push, 1)
#endif
#endif

/* Maximum size definitions */

#define SMI_MAX_MM_IP_COUNT	      8
#define SMI_MAX_DATE_LENGTH	      32 /**< YYYY-MM-DD:HH:MM:SS.MSC */
#define SMI_MAX_STRING_LENGTH	      64
#define SMI_NORMAL_STRING_LENGTH      32
#define SMI_MAX_DEVICES		      32
#define SMI_MAX_NAME		      32
#define SMI_MAX_DRIVER_VERSION_LENGTH 80
#define SMI_MAX_CONTAINER_TYPE    2

#define SMI_GPU_UUID_SIZE 38

/* string format */
#define SMI_TIME_FORMAT "%02d:%02d:%02d.%03d"
#define SMI_DATE_FORMAT "%04d-%02d-%02d:%02d:%02d:%02d.%03d"

/* <<<<<<<<<<<<<<<<<<<<<<< ENUMERATORS <<<<<<<<<<<<<<<<<<<<<<< */

enum smi_mm_ip { MM_UVD, MM_VCE, MM_VCN, MM__MAX };

enum smi_ucode_engine_id {
	UCODE_ID_SMU = 1,
	UCODE_ID_CP_CE,
	UCODE_ID_CP_PFP,
	UCODE_ID_CP_ME,
	UCODE_ID_CP_MEC_JT1,
	UCODE_ID_CP_MEC_JT2,
	UCODE_ID_CP_MEC1,
	UCODE_ID_CP_MEC2,
	UCODE_ID_RLC,
	UCODE_ID_SDMA0,
	UCODE_ID_SDMA1,
	UCODE_ID_SDMA2,
	UCODE_ID_SDMA3,
	UCODE_ID_SDMA4,
	UCODE_ID_SDMA5,
	UCODE_ID_SDMA6,
	UCODE_ID_SDMA7,
	UCODE_ID_VCN,
	UCODE_ID_UVD,
	UCODE_ID_VCE,
	UCODE_ID_ISP,
	UCODE_ID_DMCU_ERAM, /*eRAM*/
	UCODE_ID_DMCU_ISR,  /*ISR*/
	UCODE_ID_RLC_RESTORE_LIST_GPM_MEM,
	UCODE_ID_RLC_RESTORE_LIST_SRM_MEM,
	UCODE_ID_RLC_RESTORE_LIST_CNTL,
	UCODE_ID_RLC_V,
	UCODE_ID_MMSCH,
	UCODE_ID_PSP_SYSDRV,
	UCODE_ID_PSP_SOSDRV,
	UCODE_ID_PSP_TOC,
	UCODE_ID_PSP_KEYDB,
	UCODE_ID_DFC,
	UCODE_ID_PSP_SPL,
	UCODE_ID_DRV_CAP,
	UCODE_ID_MC,
	UCODE_ID_PSP_BL,
	UCODE_ID_CP_PM4,
	UCODE_ID__MAX
};

enum smi_clock_domain {
	CLOCK_DOMAIN_GFX,
	CLOCK_DOMAIN_MEM,
	CLOCK_DOMAIN_MM,
	CLOCK_DOMAIN_MM1,
	CLOCK_DOMAIN_MM2,
	CLOCK_DOMAIN__MAX
};

enum smi_thermal_domain {
	THERMAL_DOMAIN_EDGE,
	THERMAL_DOMAIN_HOTSPOT,
	THERMAL_DOMAIN_MEM,
	THERMAL_DOMAIN_PLX,
	THERMAL_DOMAIN__MAX,
};

/*
*    Flags used in conjunction with gpu_caps.supported_fields_flags to set or get appropriate bits
*
*    XGMI_FLAG represents xgmi data(xgmi_lanes, xgmi_hive_id, xgmi_node_id, xgmi_index)
*    MM_METRICS_FLAG represents mm metrics, mm_usage and mm clock data(cur_clk, avg_clk, max_clk)
*    POWER_GFX_VOLTAGE_FLAG represents gfx voltage
*    POWER_DPM_FLAG represents dpm
*    MEM_USAGE_FLAG represents mem_usage
*    MAX_FREQUENCY_TARGET_RANGE_FLAG represents ability to set soft_max frequency limit
*/

enum smi_supported_flags {
	XGMI_FLAG			= 1 << 0,
	MM_METRICS_FLAG			= 1 << 1,
	POWER_GFX_VOLTAGE_FLAG		= 1 << 2,
	POWER_DPM_FLAG			= 1 << 3,
	MEM_USAGE_FLAG			= 1 << 4,
	RESERVED_FLAG0			= 1 << 5,
	RESERVED_FLAG1			= 1 << 6,
	MAX_FREQUENCY_TARGET_RANGE_FLAG = 1 << 7,
};

enum smi_container_types {
	CONTAINER_LXC,
	CONTAINER_DOCKER,
};

/* <<<<<<<<<<<<<<<<<<<<<< AUX STRUCTURES <<<<<<<<<<<<<<<<<<<<<<<<< */
union smi_bdf {
	struct {
		uint32_t function_number : 3;
		uint32_t device_number : 5;
		uint32_t bus_number : 8;
		uint32_t domain_number : 16;
	};
	uint32_t as_uint;
};

struct smi_device_handle {
	uint64_t handle;
};

struct smi_pcie_info {
	union smi_bdf bdf;
	uint8_t	      pcie_lanes;
	uint32_t      pcie_speed;
};

struct smi_xgmi_info {
	uint8_t	 xgmi_lanes;
	uint64_t xgmi_hive_id;
	uint64_t xgmi_node_id;
	uint32_t index;
};

struct smi_bus_info {
	struct smi_pcie_info pcie;
	struct smi_xgmi_info xgmi;
};

struct smi_power_info {
	uint32_t power_cap;
	uint32_t dpm_cap;
};

/**
 * GPU Capability info
 */
struct smi_gpu_caps {
	struct {
		uint32_t gfxip_major;
		uint32_t gfxip_minor;
		uint16_t gfxip_cu_count;
	} gfx;
	struct {
		uint8_t mm_ip_count;
		uint8_t mm_ip_list[SMI_MAX_MM_IP_COUNT];
	} mm;

	bool	 ras_supported;
	uint8_t	 reserved0;
	uint32_t supported_fields_flags; /**< Supported data (xgmi, mm metrics...) flags */
	uint32_t gfx_ip_count;
	uint32_t dma_ip_count;
};

struct smi_vbios_info {
	char	 name[SMI_MAX_STRING_LENGTH];
	uint32_t vbios_version;
	char	 build_date[SMI_MAX_DATE_LENGTH];
	char	 part_number[SMI_MAX_STRING_LENGTH];
	char	 vbios_version_string[SMI_NORMAL_STRING_LENGTH];
};

struct smi_ucode_info {
	uint8_t num_ucode_info;
	struct {
		uint8_t	 ucode_id;
		uint32_t ucode_version;
	} ucode_info_list[UCODE_ID__MAX];
};

struct smi_asic_info {
	char	 market_name[SMI_MAX_STRING_LENGTH];
	uint32_t family; /**< Has zero value */
	uint32_t vendor_id;
	uint32_t device_id;
	uint32_t rev_id;
	uint64_t asic_serial;
};

struct smi_power_measure {
	uint32_t power;	      /**< Measurement in W */
	uint32_t voltage;     /**< GFX voltage measurement in mV */
	uint32_t voltage_soc; /**< SOC voltage measurement in mV */
	uint32_t voltage_mem; /**< MEM voltage measurement in mV */
	uint32_t fan_rpm;
	uint64_t energy;
	uint32_t dpm;
};

struct smi_gpu_thermal_measure {
	uint32_t temperature;
};

struct smi_gpu_clock_measure {
	uint32_t cur_clk;
	uint32_t avg_clk;
	uint32_t max_clk;
};

struct smi_engine_usage {
	uint32_t gfx_usage; /**< 0-100 */
	uint32_t mem_usage;
	uint32_t reserved0;
	uint32_t mm_ip_usage[SMI_MAX_MM_IP_COUNT];
};


typedef uint32_t smi_process_handle;

#define SMI_PROCESS_FLAGS_HAS_USAGE_METRICS   (1 << 0)
#define SMI_PROCESS_FLAGS_HAS_COMPUTE_METRICS (1 << 1)

typedef struct smi_process_info {
	char		   name[SMI_NORMAL_STRING_LENGTH];
	smi_process_handle pid;
	uint64_t	   mem; /** in bytes */
	uint32_t	   flags;
	struct {
		uint16_t gfx[SMI_MAX_MM_IP_COUNT];
		uint16_t compute[SMI_MAX_MM_IP_COUNT];
		uint16_t sdma[SMI_MAX_MM_IP_COUNT];
		uint16_t enc[SMI_MAX_MM_IP_COUNT];
		uint16_t dec[SMI_MAX_MM_IP_COUNT];
	} usage; /** percentage 0-100% times 100 */
	char		   container_name[SMI_NORMAL_STRING_LENGTH];
} smi_process_info;

#ifndef __linux__
#pragma pack(pop)
#endif

#ifdef __cplusplus
}
#endif

#endif // __SMI_LIB_TYPES_CMD_H__
